package com.yihu.ehr.framework.util;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;

public class DateUtil {

    public static final String DEFAULT_DATE_YMD_FORMAT = "yyyy-MM-dd";
    public final static String DEFAULT_YMDHMSDATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

    /**
     * 日期比较,判断endDate - fromDate 是否超过expiresIn，是则返回true，否则返回false.
     *
     *  @param fromDate
     *  @param endDate
     *  @param expiresIn 有效期间
     *
     * @return true -- 过期  false -- 未过期
     */
    public static Boolean isExpire(Date fromDate, Date endDate, Integer expiresIn){

        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        long diff = (endDate.getTime() - fromDate.getTime())/1000;

        if(diff > expiresIn){
            return true;
        }
        else{
            return false;
        }
    }

    /**
     * 日期增减,根据传入的参数，进行日期的增减.
     *
     *  @param date   需要变更的日期
     *  @param type   变更的类型 y -代码变更年份， m - 代表变更月份， d - 代表变更日期
     *  @param cos    变更的数量  正数值 增加， 负数值 减少
     *
     * @return Date
     */
    public static Date add(Date date, String type, int cos) {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);

        if(type.equals("y"))
        {
            calendar.add(Calendar.YEAR,cos);

            return calendar.getTime();
        }
        else if(type.equals("m"))
        {
            calendar.add(Calendar.MONTH,cos);

            return calendar.getTime();
        }
        else if(type.equals("d"))
        {
            calendar.add(Calendar.DATE,cos);

            return calendar.getTime();
        }
        else
        {
            return null;
        }
    }

    /**
     * 将getTime转化回DATE类型.
     *
     * @param cos 需要变更的日期
     * @return Date
     */
    public static String add(long cos) throws ParseException {

        SimpleDateFormat dfs = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date begin = dfs.parse("1970-01-01 00:00:00:00");

        java.util.Date end = new java.util.Date(cos * 1000
                + begin.getTime());
        String endStr = dfs.format(end);

        return endStr;

    }

    /**
     * 格式化时间
     * @param date
     * @return
     */
    public static String checkTodayOrYes(Date date) {
        if(date==null)
            return "-1";

        Calendar current = Calendar.getInstance();
        Calendar today = Calendar.getInstance();	//今天

        today.set(Calendar.YEAR, current.get(Calendar.YEAR));
        today.set(Calendar.MONTH, current.get(Calendar.MONTH));
        today.set(Calendar.DAY_OF_MONTH,current.get(Calendar.DAY_OF_MONTH));
        //  Calendar.HOUR——12小时制的小时数 Calendar.HOUR_OF_DAY——24小时制的小时数
        today.set( Calendar.HOUR_OF_DAY, 0);
        today.set( Calendar.MINUTE, 0);
        today.set(Calendar.SECOND, 0);

        Calendar yesterday = Calendar.getInstance();	//昨天

        yesterday.set(Calendar.YEAR, current.get(Calendar.YEAR));
        yesterday.set(Calendar.MONTH, current.get(Calendar.MONTH));
        yesterday.set(Calendar.DAY_OF_MONTH,current.get(Calendar.DAY_OF_MONTH)-1);
        yesterday.set( Calendar.HOUR_OF_DAY, 0);
        yesterday.set( Calendar.MINUTE, 0);
        yesterday.set(Calendar.SECOND, 0);

        current.setTime(date);

        if(current.after(today)){
            return "今天 ";
        }else if(current.before(today) && current.after(yesterday)){
            return "昨天 ";
        }else{
            return "-1";
        }
    }

    public static java.sql.Date formatCharDateYMD(String str) {
        return formatCharDateYMD(str, DEFAULT_DATE_YMD_FORMAT);
    }

    public static java.sql.Date formatCharDateYMD(String str, String format) {

        if (str == null || str.trim().length() == 0) {
            return null;
        }

        if (format == null) {
            throw new IllegalArgumentException("The value of an argument is inaccurate.");
        }

        SimpleDateFormat sdf = new SimpleDateFormat(format);

        ParsePosition pos = new ParsePosition(0);

        java.util.Date date = sdf.parse(str, pos);

        if (date == null) {
            return null;
        }
        return new java.sql.Date(date.getTime());
    }

    public static Timestamp getSysDateTime() {
        return new Timestamp(Calendar.getInstance().getTime().getTime());
    }

    public static Timestamp formatYMDToYMDHMS(String str) {

        String format = DEFAULT_YMDHMSDATE_FORMAT;

        if (str == null || str.trim().length() == 0) {

            return null;
        }

        str += " 00:00:00";

        SimpleDateFormat sdf = new SimpleDateFormat(format);
        ParsePosition pos = new ParsePosition(0);
        Date date = sdf.parse(str, pos);

        if (date == null) {

            return null;
        }

        Timestamp ts = DateUtil.fromatDateToTimestamp(new java.sql.Date(date.getTime()));

        return ts;
    }

    public static Timestamp fromatDateToTimestamp(Date date) {
        try {

            SimpleDateFormat df = new SimpleDateFormat(DEFAULT_YMDHMSDATE_FORMAT);
            String time = df.format(date);
            Timestamp ts = Timestamp.valueOf(time);
            return ts;
        } catch (Exception e) {

            return null;
        }
    }

    public static String getCurrentString() {
        return getCurrentString(DEFAULT_YMDHMSDATE_FORMAT);
    }

    public static String getCurrentString(String pattern) {
        SimpleDateFormat f = new SimpleDateFormat(pattern);
        return f.format(Calendar.getInstance(TimeZone.getDefault()).getTime());
    }

    public static Date addDate(int add, Date d) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(d);
        cal.add(Calendar.DATE, add);
        return formatCharToYMDHMS((new SimpleDateFormat(DateUtil.DEFAULT_YMDHMSDATE_FORMAT)).format(cal.getTime()), DateUtil.DEFAULT_YMDHMSDATE_FORMAT);
    }

    public static Timestamp formatCharToYMDHMS(String str, String format) {

        if (str == null || str.trim().length() == 0) {

            return null;
        }

        SimpleDateFormat sdf = new SimpleDateFormat(format);
        ParsePosition pos = new ParsePosition(0);
        Date date = sdf.parse(str, pos);

        if (date == null) {

            return null;
        }

        Timestamp ts = DateUtil.fromatDateToTimestamp(new Date(date.getTime()));

        return ts;
    }
}
