package com.yihu.ehr.framework.util.quartz;

/**
 * Created by hzp on 2016/2/1.
 */

import org.quartz.*;
import org.quartz.impl.StdScheduler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

import static org.quartz.JobBuilder.newJob;
import static org.quartz.TriggerBuilder.newTrigger;

/**
 * @Description: 定时任务管理类
 */
@Component
public class QuartzManager {
    private String JOB_GROUP_NAME = "DEFAULT_JOBGROUP_NAME";
    private String TRIGGER_GROUP_NAME = "DEFAULT_JOBGROUP_NAME";

    @Autowired
    private StdScheduler quartzScheduler;

    private JobDetail createJob(String jobName,Class<? extends Job> jobClass) throws Exception {
        JobDetail job = newJob(jobClass)
                .withIdentity(jobName, JOB_GROUP_NAME)
                .build();
        return job;
    }


    private Trigger createTrigger(String jobName,Date startDate, String cron) throws Exception {
        if(cron!=null&&cron.length()>0) {
            return newTrigger()
                    .startAt(startDate)
                    .withIdentity(jobName, TRIGGER_GROUP_NAME)
                    .withSchedule(CronScheduleBuilder.cronSchedule(cron).withMisfireHandlingInstructionDoNothing())
                    .build();
        }
        else{
            return newTrigger()
                    .startAt(startDate)
                    .withIdentity(jobName, TRIGGER_GROUP_NAME)
                    .build();
        }
    }

    /**
     * 通过任务类型获取任务执行类
     * @param type 0微服务任务  1类处理任务
     * @return
     */
    private Class getClassByType(String type)
    {
        if(type=="0")
        {
            return RestJob.class;
        }
        else {
            return ClassJob.class;
        }
    }





    /********************************************************************************************/
    /**
     * 启动quartz服务
     */
    public void startJobs() throws Exception {
        if(!quartzScheduler.isStarted())
        {
            quartzScheduler.start();
        }
    }

    /**
     * 关闭quartz服务
     */
    public void shutdownJobs() throws Exception {
        if(!quartzScheduler.isShutdown())
        {
            quartzScheduler.shutdown();
        }
    }

    /**
     * 添加任务
     */
    public void addJob(String jobName,String type,String content,Date startDate,String cron) throws Exception
    {
        Class cls = getClassByType(type);
        addJob(jobName,content,cls,startDate,cron);
    }

    /**
     * 添加一个定时任务
     */
    public void addJob(String jobName,String content, Class cls,Date startDate, String cron) throws Exception {
        JobDetail jobDetail = createJob(jobName, cls);
        jobDetail.getJobDataMap().put("jobId", jobName);
        jobDetail.getJobDataMap().put("jobContent", content);


        Trigger trigger = createTrigger(jobName,startDate,cron);
        quartzScheduler.scheduleJob(jobDetail, trigger);


        // 启动
        if (!quartzScheduler.isShutdown()) {
            quartzScheduler.start();
        }
    }

    /**
     * 修改一个任务的触发时间(使用默认的任务组名，触发器名，触发器组名)
     */
    public void modifyJob(String jobName,String type,String content,Date startDate, String cron) throws Exception {
        TriggerKey key =new TriggerKey(jobName, TRIGGER_GROUP_NAME);
        if (!quartzScheduler.checkExists(key)) {
            return;
        }

        removeJob(jobName);
        addJob(jobName,content, getClassByType(type),startDate, cron);

        /*Trigger trigger = sched.getTrigger(key);

        JobDetail jobDetail = sched.getJobDetail(new JobKey(jobName, JOB_GROUP_NAME));
        Class oldClass = jobDetail.getJobClass();
        String oldCron = ((CronTrigger)trigger).getCronExpression();
        String oldContent = String.valueOf(jobDetail.getJobDataMap().get("jobContent"));
        Class newClass = getClassByType(type);
        if (!oldCron.equalsIgnoreCase(cron) || newClass!=oldClass || !oldContent.equals(content)) {
            removeJob(jobName);
            addJob(jobName,content, newClass,startDate, cron);
        }*/

        // 启动
        if (!quartzScheduler.isStarted()) {
            quartzScheduler.start();
        }
    }

    /**
     * 移除一个任务
     */
    public void removeJob(String jobName) throws Exception {
        quartzScheduler.pauseTrigger(new TriggerKey(jobName, TRIGGER_GROUP_NAME));// 停止触发器
        quartzScheduler.unscheduleJob(new TriggerKey(jobName, TRIGGER_GROUP_NAME));// 移除触发器
        quartzScheduler.deleteJob(new JobKey(jobName, JOB_GROUP_NAME));// 删除任务
    }

    /**
     * 暂停任务
     */
    public void pauseJob(String jobName) throws Exception {
        quartzScheduler.pauseJob(new JobKey(jobName, JOB_GROUP_NAME));
        quartzScheduler.pauseTrigger(new TriggerKey(jobName, TRIGGER_GROUP_NAME));
    }

    /**
     * 恢复任务
     */
    public void resumeJob(String jobName) throws Exception {
        quartzScheduler.resumeJob(new JobKey(jobName, JOB_GROUP_NAME));
        quartzScheduler.resumeTrigger(new TriggerKey(jobName, TRIGGER_GROUP_NAME));
    }

}
